package ru.ulstu.is.sbapp.HardwareShop.services;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;
import ru.ulstu.is.sbapp.HardwareShop.controller.ProductDTO;
import ru.ulstu.is.sbapp.HardwareShop.models.Manufacturer;
import ru.ulstu.is.sbapp.HardwareShop.models.Product;

import javax.persistence.EntityManager;
import javax.persistence.EntityNotFoundException;
import javax.persistence.PersistenceContext;
import org.springframework.transaction.annotation.Transactional;
import ru.ulstu.is.sbapp.HardwareShop.repository.ManufacturerRepository;
import ru.ulstu.is.sbapp.HardwareShop.repository.ProductRepository;
import ru.ulstu.is.sbapp.Util.Validation.ValidatorUtil;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.Optional;

@Service
public class ProductService {
    @Autowired
    private final ProductRepository productRepository;

    @Autowired
    private final ManufacturerService manufacturerService;

    @Autowired
    private final CategoryService categoryService;

    @Autowired
    private final ValidatorUtil validatorUtil;

    public ProductService(ProductRepository productRepository, ValidatorUtil validatorUtil, ManufacturerService manufacturerService, CategoryService categoryService) {
        this.productRepository = productRepository;
        this.validatorUtil = validatorUtil;
        this.manufacturerService = manufacturerService;
        this.categoryService = categoryService;
    }

    @Transactional
    public Product addProduct(ProductDTO productDTO) throws IOException {
        final Product product = new Product(productDTO);
        product.setCategory(categoryService.findCategory(productDTO.getCategory_id()));
        validatorUtil.validate(product);
        return productRepository.save(product);
    }

    @Transactional
    public Product addProduct(Integer price, byte[] image, String name) {
        final Product product = new Product(price, image, name);
        validatorUtil.validate(product);
        return productRepository.save(product);
    }

    @Transactional(readOnly = true)
    public Product findProduct(Long id) {
        final Optional<Product> post = productRepository.findById(id);
        return post.orElseThrow(() -> new ProductNotFoundException(id));
    }

    @Transactional
    public void saveProduct(Product product) {
        productRepository.save(product);
    }

    @Transactional(readOnly = true)
    public List<Product> findAllProducts() {
        return productRepository.findAll();
    }

    @Transactional
    public Product updateProduct(Long id, ProductDTO productDTO) {
        final Product currentProduct = findProduct(id);
        currentProduct.setPrice(productDTO.getPrice());
        /*currentProduct.setPhoto(productDTO.getPhoto().getBytes(StandardCharsets.UTF_8));*/
        validatorUtil.validate(currentProduct);
        return productRepository.save(currentProduct);
    }

    @Transactional
    public Product updateProduct(Long id, Integer price, byte[] image) {
        final Product currentProduct = findProduct(id);
        currentProduct.setPrice(price);
        /*currentProduct.setPhoto(image);*/
        validatorUtil.validate(currentProduct);
        return productRepository.save(currentProduct);
    }

    @Transactional
    public Product deleteProduct(Long id) {
        final Product currentProduct = findProduct(id);
        productRepository.delete(currentProduct);
        return currentProduct;
    }

    @Transactional
    public void deleteAllProducts() {
        productRepository.deleteAll();
    }

    @Transactional
    public void addManufacturersToProduct(Long id, Long manufacturer_id) {
        final Product currentProduct = findProduct(id);
        final Manufacturer manufacturer = manufacturerService.findManufacturer(manufacturer_id);
        currentProduct.addManufacturer(manufacturer);
        validatorUtil.validate(currentProduct);
        productRepository.save(currentProduct);
    }

    @Transactional
    public void removeManufacturersToProduct(Long id, Long manufacturer_id) {
        final Product currentProduct = findProduct(id);
        currentProduct.removeManufacturer(manufacturerService.findManufacturer(manufacturer_id));
        validatorUtil.validate(currentProduct);
        productRepository.save(currentProduct);
    }

    @Transactional
    public List getProductManufacturers(Long id){
        List manufacturers = productRepository.getManufacturers(id);
        return manufacturers;
    }
}
